@@ -1,6 +1,6 @@
 package Apache::PageKit::View;
 
-# $Id: View.pm,v 1.53 2001/05/16 22:13:41 tjmather Exp $
+# $Id: View.pm,v 1.87 2002/03/22 23:12:04 tjmather Exp $
 
 # we want to extend this module to use different templating packages -
 # Template::ToolKit and HTML::Template
@@ -12,14 +12,10 @@ use File::Find ();
 use File::Path ();
 use HTML::Clean ();
 
-use XML::XPathTemplate ();
+use HTML::Template::XPath ();
 
 use Storable ();
 
-use Carp ();
-
-use Data::Dumper;
-
 # how loading, filter and caching works on the templates.
 # 1. templates are pre-filtered, to convert MODEL_*,VIEW_* and PKIT_* tags
 # corresponding TMPL_ tags, and to run HTML::Clean
@@ -28,7 +24,7 @@ use Data::Dumper;
 # a hash containing fields from the following set:
 #    * exclude_params - array ref of lists of params to be excluded from <PKIT_SELFURL> tags
 #    * html_template - HTML::Template object
-#    * template_toolkit - Template-Tookit object
+#    * template_toolkit - Template-Tookit object (NOT USED NOW)
 #    * filename - filename of template source
 #    * include_mtimes - a hash ref with file names as keys and mtimes as values
 #        (contains all of the files included by the <PKIT_COMPONENT> tags
@@ -53,9 +49,36 @@ use Data::Dumper;
 #    * _fill_in_content_loop(...)
 #    * _load_page($page_id, $pkit_view, [$template_file])
 
+
+
+# these global vars are initialised and then they are readonly!
+# this is done here mainly for speed.
+use vars qw /%replace_start_tags %replace_end_tags $key_value_pattern/;
+
+%replace_start_tags = (
+                               MESSAGES  => '<TMPL_LOOP NAME="PKIT_MESSAGES">',
+                               IS_ERROR  => '<TMPL_IF NAME="PKIT_IS_ERROR">',
+                               NOT_ERROR => '<TMPL_UNLESS NAME="PKIT_IS_ERROR">',
+                               HOSTNAME  => '<TMPL_VAR NAME="PKIT_HOSTNAME">',
+                               MESSAGE   => '<TMPL_VAR NAME="PKIT_MESSAGE">',
+                               ERRORSTR  => '<TMPL_VAR NAME="PKIT_ERRORSTR">',
+                               REALURL   => '<TMPL_VAR NAME="PKIT_REALURL">',
+  );
+
+%replace_end_tags = (
+                             VIEW      => '</TMPL_IF>',
+                             IS_ERROR  => '</TMPL_IF>',
+                             NOT_ERROR => '</TMPL_UNLESS>',
+                             MESSAGES  => '</TMPL_LOOP>'
+  );
+
+#                        --------------------- $1 --------------------------
+#                             $2                  $3         $4       $5
+$key_value_pattern = qr!(\s+(\w+)(?:\s*=\s*(?:"([^"]*)"|\'([^\']*)\'|(\w+)))?)!;    #"
+
 $Apache::PageKit::View::cache_component = {};
 
-sub new($$) {
+sub new {
   my $class = shift;
   my $self = { @_ };
   bless $self, $class;
@@ -79,6 +102,9 @@ sub fill_in_view {
     foreach my $exclude_params (@$exclude_params_set){
       my @exclude_params = split(" ",$exclude_params);
       my $query_string = Apache::PageKit::params_as_string($input_param_object, \@exclude_params);
+
+      #remove empty parameters as arised from http://ka.zyx.de/galerie?show=abc& or <PKIT_SELFURL>
+      $query_string =~ s!=&!!;
       if($query_string){
 	$tmpl->param("pkit_selfurl$exclude_params", ($orig_uri . '?' . $query_string) . '&');
       } else {
@@ -92,7 +118,8 @@ sub fill_in_view {
     foreach my $key ($object->param){
       # note that we only fill in MODEL_VARs, to avoid errors when setting
       # loops in HTML::Template
-      if ($tmpl->query(name => $key) eq 'VAR'){
+      my $type = $tmpl->query(name => $key);
+      if ( $type && $type eq 'VAR' ) {
 	$tmpl->param($key,$object->param($key));
       }
     }
@@ -115,17 +142,47 @@ sub fill_in_view {
     my $fif;
     if(@{$view->{fillinform_objects}}){
       $fif = HTML::FillInForm->new();
-      $output = $fif->fill(scalarref=>\$output,
-			   fobject => $view->{fillinform_objects} );
+      $output = $fif->fill(scalarref => \$output,
+                           fobject   => $view->{fillinform_objects} );
     }
   }
-
-  if(exists $INC{'Apache/PKCMS/View.pm'}){
-    Apache::PKCMS::View::add_edit_links($view, $record, \$output);
+  if($view->{can_edit} eq 'yes'){
+    Apache::PageKit::Edit::add_edit_links($view, $record, \$output);
   }
   return \$output;
 }
 
+# gets static gzipped file, creating it if necessary
+sub get_static_gzip {
+  my ($view, $filename) = @_;
+  my ($gzip_mtime, $gzipped_content);
+
+  (my $relative_filename = $filename) =~ s!^$view->{view_dir}/!!;
+  my $gzipped_filename = "$view->{cache_dir}/$relative_filename.gz";
+
+  # is the cache entry valid or changed on disc?
+  if(-f "$gzipped_filename"){
+    open FH, "<$gzipped_filename" or return undef;
+    # read mtime from first line
+    chomp($gzip_mtime = <FH>);
+
+    # read rest of gzipped content
+    local $/ = undef;
+    $gzipped_content = <FH>;
+    close FH;
+    if($view->{reload} ne 'no'){
+      # is the cache entry valid or changed on disc?
+      my $mtime = ( stat($filename) )[9];
+      if($mtime != $gzip_mtime){
+	$gzipped_content = $view->_create_static_zip($filename, $gzipped_filename);
+      }
+    }
+  } else {
+    $gzipped_content = $view->_create_static_zip($filename, $gzipped_filename);
+  }
+  return $gzipped_content;
+}
+
 # opens template, each 
 sub open_view {
   my ($view, $page_id, $pkit_view, $lang) = @_;
@@ -141,7 +198,7 @@ sub open_view {
       unless $record;
   }
 
-  if($view->{reload} ne "no"){
+  if($view->{reload} ne 'no'){
     # check for updated files on disk
     unless($view->_is_record_uptodate($record, $pkit_view, $page_id)){
       # one of the included files changed on disk, reload
@@ -180,17 +237,50 @@ sub template_file_exists {
 
 # private methods
 
+# creates gzipped file
+sub _create_static_zip {
+  my ($view, $filename, $gzipped_filename) = @_;
+  local $/ = undef;
+  open FH, "<$filename" or return undef;
+  my $content = <FH>;
+  close FH;
+
+  $view->_html_clean(\$content);
+
+  my $gzipped_content = Compress::Zlib::memGzip($content);
+
+  (my $gzipped_dir = $gzipped_filename) =~ s!(/)?[^/]*?$!!;
+
+  File::Path::mkpath("$gzipped_dir");
+
+  if ($gzipped_content) {
+    my $mtime = (stat($filename))[9];
+    if ( open GZIP, ">$gzipped_filename" ) {
+      print GZIP "$mtime\n";
+      print GZIP $gzipped_content;
+      close GZIP;
+    } else {
+      warn "can't create gzip cache file $view->{cache_dir}/$gzipped_filename: $!";
+    }
+    return $gzipped_content;
+  }
+  return undef;
+}
+
 sub _fetch_from_file_cache {
   my ($view, $page_id, $pkit_view, $lang) = @_;
 
-  my ($extra_param, $param_hash) = ("","");
-  if (my @xml_params = sort keys %{$Apache::PageKit::Content::PAGE_ID_XSL_PARAMS->{$page_id}}){
-    my $param_obj = $view->{input_param_object};
-    for my $xml_param (@xml_params){
-      $extra_param .= "&$xml_param=" . $param_obj->param($xml_param);
+    my ($extra_param, $param_hash) = ("", "");
+    
+    # get a list of requested params in the *.xsl file
+    if (my @xml_params = sort keys %{$Apache::PageKit::Content::PAGE_ID_XSL_PARAMS->{$page_id}}) {
+      my $param_obj = $view->{input_param_object};
+      for my $xml_param (@xml_params){
+        my $value = $param_obj->param($xml_param) || '';
+	$extra_param .= "&$xml_param=" . $value;
+      }
+      $param_hash = Digest::MD5::md5_hex($extra_param);
     }
-    $param_hash = Digest::MD5->md5_hex($extra_param);
-  }
 
   my $cache_filename = "$view->{cache_dir}/$page_id.$pkit_view.$lang$param_hash";
 
@@ -211,11 +301,11 @@ sub _fetch_from_file_cache {
 sub _find_template {
   my ($view, $pkit_view, $id) = @_;
   my $template_file = "$view->{view_dir}/$pkit_view/$id.tmpl";
-  if(-e "$template_file"){
+  if(-f "$template_file"){
     return $template_file;
   } else {
     $template_file = "$view->{view_dir}/Default/$id.tmpl";
-    if(-e "$template_file"){
+    if(-f "$template_file"){
       return $template_file;
     } else {
       return undef;
@@ -232,7 +322,7 @@ sub _html_clean {
   return unless $html_clean_level > 0;
 
   my $h = new HTML::Clean($html_code_ref,$html_clean_level) || die("can't open HTML::Clean object: $!");
-  $h->strip if $html_clean_level > 0;
+  $h->strip;
   $$html_code_ref = ${$h->data()};
 }
 
@@ -240,13 +330,20 @@ sub _html_clean {
 sub _include_components {
   my ($view, $page_id, $html_code_ref, $pkit_view) = @_;
 
-  $$html_code_ref =~ s!<PKIT_COMPONENT (NAME=)?"?(.*?)"?>(</PKIT_COMPONENT>)?!&get_component($page_id,$2,$view,$pkit_view)!eig;
-
-#  my @component_ids = keys %component_ids;
-#  return \@component_ids;
-
+  if ( $view->{relaxed_parser} eq 'yes' ) {
+    $$html_code_ref =~ s%<(!--)?\s*PKIT_COMPONENT($key_value_pattern+)\s*/?(?(1)--)?>(?:<(!--)?\s*/PKIT_COMPONENT\s*(?(1)--)>)?%get_component($page_id,$2,$view,$pkit_view)%eig;
+  } else {
+    $$html_code_ref =~ s%<\s*PKIT_COMPONENT($key_value_pattern+)\s*/?>(<\s*/PKIT_COMPONENT\s*>)?%&get_component($page_id,$1,$view,$pkit_view)%eig;
+  }
   sub get_component {
-    my ($page_id,$component_id, $view, $pkit_view) = @_;
+    my ($page_id, $params, $view, $pkit_view) = @_;
+    my %params = ();
+
+    while($params =~ m!$key_value_pattern!ig) {
+      $params{uc($2)} = $+;
+    }
+
+    my $component_id = delete $params{NAME} or die qq{component item "NAME=..." not found};
 
     unless($component_id =~ s!^/!!){
       # relative component, component relative to page_id
@@ -261,7 +358,8 @@ sub _include_components {
       die "Likely recursive PKIT_COMPONENTS for component_id $component_id and giving up.";
     }
 
-    my $template_ref = $view->_load_component($page_id,$component_id,$pkit_view);
+    my $template_ref = $view->_load_component($page_id, $component_id, $pkit_view, \%params);
+    $$template_ref =~ s!<\s*PKIT_MACRO$key_value_pattern\s*/?>!$params{uc($+)}!egi if (keys %params);
     return $$template_ref;
   }
 }
@@ -273,7 +371,7 @@ sub _is_record_uptodate {
   my $include_mtimes = $record->{include_mtimes};
   while (my ($filename, $cache_mtime) = each %$include_mtimes){
     # check if file still exists
-    unless(-e "$filename"){
+    unless(-f "$filename"){
       return 0;
     }
 
@@ -287,7 +385,7 @@ sub _is_record_uptodate {
     if($filename =~ m!^$view->{view_dir}/Default/! && $pkit_view ne 'Default'){
       # check to see if any new files have been uploaded to the $pkit_view dir
       (my $check_filename = $filename) =~ s!^$view->{view_dir}/Default/!$view->{view_dir}/$pkit_view/!;
-      if (-e "$check_filename"){
+      if (-f "$check_filename"){
 	return 0;
       }
     }
@@ -299,7 +397,7 @@ sub _is_record_uptodate {
 
 # here the usage of "component" also includes page
 sub _load_component {
-  my ($view, $page_id, $component_id, $pkit_view) = @_;
+  my ($view, $page_id, $component_id, $pkit_view, $component_params) = @_;
 
   my $template_file = $view->_find_template($pkit_view, $component_id);
   my $template_ref;
@@ -307,22 +405,33 @@ sub _load_component {
   unless($template_file){
     # no template file exists, attempt to generate from XML and XSL files
     # currently only XML::LibXSLT is supported
-    $template_ref = $view->{content}->generate_template($page_id, $component_id, $pkit_view, $view->{input_param_object});
+    $template_ref = $view->{content}->generate_template($page_id, $component_id, $pkit_view, $view->{input_param_object}, $component_params);
   } else {
-    open TEMPLATE, "$template_file";
+    open TEMPLATE, "<$template_file" or die "can not read $template_file";
     local($/) = undef;
     my $template = <TEMPLATE>;
-    $template_ref = \$template;
     close TEMPLATE;
 
+    my $default_input_charset = $view->{default_input_charset};
+    unless($default_input_charset eq 'UTF-8'){
+      my $converter;
+      eval {
+	$converter = Text::Iconv->new($default_input_charset, 'UTF-8');
+      };
+      if ($@) {
+	(my $config_dir = $view->{content_dir}) =~ s!/Content$!/Config!;
+	die "charset ($default_input_charset) is not supported by Text::Iconv please check file ${config_dir}/Config.xml";
+      }
+      $template = $converter->convert($template) || die "Can not convert page from $default_input_charset to utf-8";
+    }
+    $template_ref = \$template;
+
     my $mtime = (stat(_))[9];
     $view->{include_mtimes}->{$template_file} = $mtime;
   }
 
   if($view->{can_edit} eq 'yes'){
-    if(exists $INC{'Apache/PKCMS/View.pm'}){
-      Apache::PKCMS::View::add_component_edit_stubs($template_ref);
-    }
+    Apache::PageKit::Edit::add_component_edit_stubs($view, $template_ref, $pkit_view);
   }
 
   $view->_include_components($page_id,$template_ref,$pkit_view);
@@ -336,28 +445,58 @@ sub _load_page {
   my $content = $view->{content} ||= Apache::PageKit::Content->new(
 						     content_dir => $view->{content_dir},
 						     view_dir => $view->{view_dir},
-						     default_lang => $view->{default_lang});
+						     default_lang => $view->{default_lang},
+                                                     relaxed_parser => $view->{relaxed_parser},
+                                                     template_class => $view->{template_class},
+                                                     );
 
   $view->{lang_tmpl} = $content->{lang_tmpl} = {};
-  $view->{include_mtimes} = $content->{include_mtimes} = {};
+  $content->{include_mtimes} = {};
   $view->{component_ids_hash} = {};
 
+  # we add Config.xml to the hash of files to be checked for mtimes,
+  # in case default_input_charset or default_output_charset changes!
+  (my $config_file = $view->{view_dir}) =~ s!/View$!/Config/Config.xml!;
+  my $config_mtime = ( stat($config_file) )[9];
+  $view->{include_mtimes} = {$config_file => $config_mtime};
+
   my $template_file = $view->_find_template($pkit_view, $page_id);
   my $template_ref = $view->_load_component($page_id,$page_id,$pkit_view);
 
+  my $default_output_charset = $view->{default_output_charset};
+  my $converter;
+  unless ('UTF-8' eq $default_output_charset) {
+    eval {
+      $converter = Text::Iconv->new( 'UTF-8', $default_output_charset);
+    };
+    if ($@) {
+      (my $config_dir = $view->{content_dir}) =~ s!/Content$!/Config!;
+      die "The charset ($default_output_charset) is not supported by Text::Iconv please check file ${config_dir}/Config.xml";
+    }
+  }
+
 #  my $template_file = $view->_find_template($pkit_view, $page_id);
   my $lang_tmpl = $content->process_template($page_id, $template_ref);
 
+  # add used content file(s) to the mtimes hash
+  while( my ( $file, $mtime ) = each( %{ $content->{include_mtimes} } ) ) {
+    $view->{include_mtimes}->{$file} = $mtime;
+  }
+
   # go through content files (which have had content filled in)
   while (my ($lang, $filtered_html) = each %$lang_tmpl){
+
+    if ( $converter ) {
+      $$filtered_html = $converter->convert($$filtered_html) || die "Can not convert page from UTF-8 to $default_output_charset";
+    }
+    
     my $exclude_params_set = $view->_preparse_model_tags($filtered_html);
     $view->_html_clean($filtered_html);
 
     my $has_form = ($$filtered_html =~ m!<form!i);
-
     my $tmpl;
     eval {
-      $tmpl = HTML::Template->new(scalarref => $filtered_html,
+      $tmpl =  $view->{template_class}->new(scalarref => $filtered_html,
 					   # don't die when we set a parameter that is not in the template
 					   die_on_bad_params=>0,
 					   # built in __FIRST__, __LAST__, etc vars
@@ -366,7 +505,7 @@ sub _load_page {
 					   global_vars=>1);
     };
     if($@){
-      die "Can't load template (postprocessing) for $page_id: $@"
+      die "Can't load template (MODEL TAGS) for $page_id: $@"
     }
     my @component_ids = keys %{$view->{component_ids_hash}};
     my $record = {
@@ -386,12 +525,15 @@ sub _load_page {
     }
 
     my ($extra_param, $param_hash) = ("", "");
-    if (my @xml_params = sort keys %{$Apache::PageKit::Content::PAGE_ID_XSL_PARAMS->{$page_id}}){
+    # get a list of requested params in the *.xsl file
+    if (my @xml_params = sort keys %{$Apache::PageKit::Content::PAGE_ID_XSL_PARAMS->{$page_id}}) {
       my $param_obj = $view->{input_param_object};
+
       for my $xml_param (@xml_params){
-	$extra_param .= "&$xml_param=" . $param_obj->param($xml_param);
+        my $value = $param_obj->param($xml_param) || '';
+	$extra_param .= "&$xml_param=" . $value;
       }
-      $param_hash = Digest::MD5->md5_hex($extra_param);
+      $param_hash = Digest::MD5::md5_hex($extra_param);
     }
 
     # Store record
@@ -406,48 +548,90 @@ sub _load_page {
 }
 
 sub _preparse_model_tags {
-  my ($view, $html_code_ref) = @_;
+  my ( $view, $html_code_ref ) = @_;
 
   my $exclude_params_set = {};
 
   # "compile" PageKit templates into HTML::Templates
-  $$html_code_ref =~ s!<MODEL_(VAR|LOOP|IF|ELSE|UNLESS)!<TMPL_$1!sig;
-  $$html_code_ref =~ s!</MODEL_(LOOP|IF|UNLESS)!</TMPL_$1!sig;
+  if ( $view->{relaxed_parser} eq 'yes' ) {
 
-  # tags generated by XSLT
-  $$html_code_ref =~ s!</(MODEL|PKIT)_VAR>!!ig;
+    # new parser
 
-  $$html_code_ref =~ s!<PKIT_SELFURL( +exclude=('|")(.*?)('|"))? *>!&process_selfurl_tag($exclude_params_set, $3)!eig;
+    # the new parser is a lot more flexible over the old one. it can parse
 
-  $$html_code_ref =~ s!<PKIT_ERRORFONT (NAME=)?"?([^"]*?)"?>(.*?)</PKIT_ERRORFONT>!<TMPL_VAR NAME="PKIT_ERRORFONT_BEGIN_$2">$3<TMPL_VAR NAME="PKIT_ERRORFONT_END_$2">!sig;
-  $$html_code_ref =~ s!<PKIT_HOSTNAME>!<TMPL_VAR NAME="PKIT_HOSTNAME">!ig;
+    # <MODEL_VAR NAME=abc>
+    # <MODEL_VAR NAME=abc/>
+    # <MODEL_VAR NAME=abc   />
+    # <   MODEL_VAR NAME=abc   >
+    # <!--   MODEL_VAR NAME=abc   -->
+    # <!--MODEL_VAR NAME=abc   -->
+    # <!--   MODEL_VAR NAME=abc   /-->
 
-  $$html_code_ref =~ s!<PKIT_MESSAGES>!<TMPL_LOOP NAME="PKIT_MESSAGES">!ig;
-  $$html_code_ref =~ s!<PKIT_IS_ERROR>!<TMPL_IF NAME="PKIT_IS_ERROR">!ig;
-  $$html_code_ref =~ s!</PKIT_IS_ERROR>!</TMPL_IF>!ig;
-  $$html_code_ref =~ s!<PKIT_MESSAGE>!<TMPL_VAR NAME="PKIT_MESSAGE">!ig;
-  $$html_code_ref =~ s!</PKIT_MESSAGES>!</TMPL_LOOP>!ig;
+    # all these are valid and expanded. it is slower than the old one but if it works relaible
 
-  $$html_code_ref =~ s!<PKIT_VIEW +NAME *= *('|")?(.*?)('|")? *>!<TMPL_IF NAME="PKIT_VIEW:$2">!sig;
-  $$html_code_ref =~ s!</PKIT_VIEW>!</TMPL_IF>!ig;
+    if ( $$html_code_ref =~ m%<(!--)?\s*PKIT_(?:VAR|LOOP|IF|UNLESS)(?:$key_value_pattern)*\s*/?(?(1)--)>%i ) {
+      warn "PKIT_VAR, PKIT_LOOP, PKIT_IF, and PKIT_UNLESS are depreciated.  use PKIT_HOSTNAME, PKIT_VIEW, PKIT_MESSAGES, PKIT_IS_ERROR, PKIT_NOT_ERROR or PKIT_MESSAGE instead";
+    }
 
-  if($$html_code_ref =~ m!<PKIT_(VAR|LOOP|IF|UNLESS) (.*?)>!){
-    warn "PKIT_VAR, PKIT_LOOP, PKIT_IF, and PKIT_UNLESS are depreciated.  use PKIT_HOSTNAME, PKIT_VIEW, PKIT_MESSAGES, PKIT_IS_ERROR, or PKIT_MESSAGE instead";
+    # remove tags
+    # tags generated by XSLT
+    $$html_code_ref =~ s%<(!--)?\s*/(?:MODEL|PKIT)_VAR\s*(?(1)--)>%%sig;
+
+    # translate end to tmpl
+    $$html_code_ref =~ s%<(!--)?\s*/(?:MODEL|PKIT)_(LOOP|IF|UNLESS)\s*(?(1)--)>%</TMPL_$2>%sig;
+
+    # XML-style stand-alone tags and other start tags
+    $$html_code_ref =~ s%<(!--)?\s*(?:MODEL|PKIT)_(VAR|LOOP|IF|ELSE|UNLESS)($key_value_pattern*)\s*/?(?(1)--)>%<TMPL_$2$3>%sig;
+
+    $$html_code_ref =~
+      s%<(!--)?\s*PKIT_SELFURL$key_value_pattern?\s*/?(?(1)--)>% &process_selfurl_tag($exclude_params_set, $4 || $5 || $6 || $3 ) %seig;
+
+    $$html_code_ref =~ s%<(!--)?\s*/PKIT_(VIEW|IS_ERROR|NOT_ERROR|MESSAGES)\s*(?(1)--)>%     $replace_end_tags{uc($2)}   %seig;
+    $$html_code_ref =~ s%<(!--)?\s*PKIT_(MESSAGES|IS_ERROR|NOT_ERROR)\s*(?(1)--)>%           $replace_start_tags{uc($2)} %seig;
+    $$html_code_ref =~ s%<(!--)?\s*PKIT_(HOSTNAME|MESSAGE|ERRORSTR|REALURL)\s*/?(?(1)--)>% $replace_start_tags{uc($2)} %seig;
+
+    $$html_code_ref =~
+      s^<(!--)?\s*PKIT_VIEW$key_value_pattern\s*/?(?(1)--)>^ sprintf '<TMPL_IF NAME="PKIT_VIEW:%s">', $4 || $5 || $6 || $3; ^sieg; #"
+
+    $$html_code_ref =~
+      s^<(!--)?\s*PKIT_ERROR(?:FONT|SPAN)$key_value_pattern(?(1)--)\s*>(.*?)<(!--)?\s*/PKIT_ERROR(?:FONT|SPAN)\s*(?(8)--)>^ my $font = $4 || $5 || $6 || $3; qq{<TMPL_VAR NAME="PKIT_ERRORSPAN_BEGIN_$font">$7<TMPL_VAR NAME="PKIT_ERRORSPAN_END_$font">}; ^seig;
   }
+  else {
 
-#  $$html_code_ref =~ s!<PKIT_(VAR|LOOP|IF|UNLESS) +NAME *= *("?)__(FIRST|INNER|ODD|LAST)!<TMPL_$1 NAME=$2__$3!sig;
-#  $$html_code_ref =~ s!<PKIT_(VAR|LOOP|IF|UNLESS) +NAME *= *("?)!<TMPL_$1 NAME=$2PKIT_!sig;
-  $$html_code_ref =~ s!<PKIT_ELSE!<TMPL_ELSE!sig;
-  $$html_code_ref =~ s!</PKIT_(LOOP|IF|UNLESS)!</TMPL_$1!sig;
+      if ( $$html_code_ref =~ m%<PKIT_(?:VAR|LOOP|IF|UNLESS)(?:$key_value_pattern)*/?>%i ) {
+      warn "PKIT_VAR, PKIT_LOOP, PKIT_IF, and PKIT_UNLESS are depreciated.  use PKIT_HOSTNAME, PKIT_VIEW, PKIT_MESSAGES, PKIT_IS_ERROR, PKIT_NOT_ERROR or PKIT_MESSAGE instead";
+    }
+
+    # remove tags
+    # tags generated by XSLT
+    $$html_code_ref =~ s%</(?:MODEL|PKIT)_VAR>%%sig;
+
+    # translate end to tmpl
+    $$html_code_ref =~ s%</(?:MODEL|PKIT)_(LOOP|IF|UNLESS)>%</TMPL_$1>%sig;
+
+    # XML-style stand-alone tags and other start tags
+    $$html_code_ref =~ s%<(?:MODEL|PKIT)_(VAR|LOOP|IF|ELSE|UNLESS)($key_value_pattern*)/?>%<TMPL_$1$2>%sig;
+
+    $$html_code_ref =~
+      s%<PKIT_SELFURL$key_value_pattern?/?>% &process_selfurl_tag($exclude_params_set, $3 || $4 || $5 || $2 ) %seig;
+
+    $$html_code_ref =~ s%</PKIT_(VIEW|IS_ERROR|NOT_ERROR|MESSAGES)>%    $replace_end_tags{uc($1)}   %seig;
+    $$html_code_ref =~ s%<PKIT_(MESSAGES|IS_ERROR|NOT_ERROR)>%          $replace_start_tags{uc($1)}%seig;
+    $$html_code_ref =~ s%<PKIT_(HOSTNAME|MESSAGE|ERRORSTR|REALURL)/?>%$replace_start_tags{uc($1)}%seig;
+
+    $$html_code_ref =~
+      s^<PKIT_VIEW$key_value_pattern/?>^ sprintf '<TMPL_IF NAME="PKIT_VIEW:%s">', $3 || $4 || $5 || $2; ^sieg; #"
+    $$html_code_ref =~
+      s^<PKIT_ERROR(?:FONT|SPAN)$key_value_pattern>(.*?)</PKIT_ERROR(?:FONT|SPAN)>^ my $font = $3 || $4 || $5 || $2; qq{<TMPL_VAR NAME="PKIT_ERRORSPAN_BEGIN_$font">$6<TMPL_VAR NAME="PKIT_ERRORSPAN_END_$font">}; ^seig;
+  }
 
   my @a = keys %$exclude_params_set;
   return \@a;
 
   sub process_selfurl_tag {
-    my ($exclude_params_set, $exclude_params) = @_;
-    $exclude_params = defined($exclude_params) ? 
-      join(" ",sort split("\s+",$exclude_params)) : "";
-    %$exclude_params_set->{$exclude_params} = 1;
+    my ( $exclude_params_set, $exclude_params ) = @_;
+    $exclude_params = defined($exclude_params) ? join ( " ", sort split ( /\s+/, $exclude_params ) ) : "";
+    $exclude_params_set->{$exclude_params} = 1;
     return qq{<TMPL_VAR NAME="pkit_selfurl$exclude_params">};
   }
 }
